class GameConstants:

    # the maximum number of rounds, until the winner is decided by a coinflip
    MAX_ROUNDS = 500

    # the board size
    BOARD_SIZE = 16

    # the default seed
    DEFAULT_SEED = 1337
